@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2015, PyRETIS Development Team.
+# Copyright (c) 2019, PyRETIS Development Team.
 # Distributed under the LGPLv2.1+ License. See LICENSE for more info.
 """A GROMACS external MD integrator interface.
 
@@ -16,13 +16,14 @@ import os
 import shlex
 import subprocess
 from time import sleep
-import numpy as np
 from pyretis.core.box import box_matrix_to_list
 from pyretis.engines.gromacs import GromacsEngine
-from pyretis.inout.writers.gromacsio import (read_trr_header,
-                                             read_trr_data,
-                                             TRR_DATA_ITEMS)
-logger = logging.getLogger(__name__)  # pylint: disable=C0103
+from pyretis.inout.formats.gromacs import (
+    read_trr_header,
+    read_trr_data,
+    TRR_DATA_ITEMS
+)
+logger = logging.getLogger(__name__)  # pylint: disable=invalid-name
 logger.addHandler(logging.NullHandler())
 
 TRR_HEAD_SIZE = 1000
@@ -32,39 +33,20 @@ TRR_HEAD_SIZE = 1000
 
 
 class GromacsEngine2(GromacsEngine):
-    """A class for interfacing GROMACS.
+    """
+    A class for interfacing GROMACS.
 
-    This class defines the interface to GROMACS.
-
-    Attributes
-    ----------
-    gmx : string
-        The command for executing GROMACS. Note that we are assuming
-        that we are using version 5 of GROMACS.
-    mdrun : string
-        The command for executing GROMACS mdrun. In some cases this
-        executable can be different from ``gmx mdrun``.
-    mdrun_c : string
-        The command for executing GROMACS mdrun when continuing a
-        simulation. This is derived from the ``mdrun`` command.
-    input_path : string
-        The directory where the input files are stored.
-    input_files : dict of strings
-        The names of the input files. We expect to find the keys
-        ``'conf'``, ``'input'`` ``'topology'``.
-    ext_time : float
-        The time to extend simulations by. It is equal to
-        ``timestep * subcycles``.
-    maxwarn : integer
-        Setting for the GROMACS grompp ``maxwarn` option.
-    ext : string
-        This string selects the output format for GROMACS.
+    This class defines an interface to GROMACS. Attributes are similar
+    to :py:class:`.GromacsEngine`. In this particular interface,
+    GROMACS is executed without starting and stopping and we rely on
+    reading the output TRR file from GROMACS while a simulation is
+    running.
     """
 
     def __init__(self, gmx, mdrun, input_path, timestep, subcycles,
                  maxwarn=0, gmx_format='g96', write_vel=True,
                  write_force=False):
-        """Initiate the GROMACS engine.
+        """Set up the GROMACS engine.
 
         Parameters
         ----------
@@ -78,18 +60,24 @@ class GromacsEngine2(GromacsEngine):
             The time step used in the GROMACS MD simulation.
         subcycles : integer
             The number of steps each GROMACS MD run is composed of.
-        maxwarn : integer
-            Setting for the GROMACS grompp ``maxwarn` option.
-        gmx_format : string
+        maxwarn : integer, optional
+            Setting for the GROMACS ``grompp -maxwarn`` option.
+        gmx_format : string, optional
             The format used for GROMACS configurations.
+        write_vel : boolean, optional
+            Determines if GROMACS should write velocities or not.
+        write_force : boolean, optional
+            Determines if GROMACS should write forces or not.
+
         """
         super().__init__(gmx, mdrun, input_path, timestep, subcycles,
                          maxwarn=maxwarn, gmx_format=gmx_format,
                          write_vel=write_vel, write_force=write_force)
 
     def _propagate_from(self, name, path, system, order_function, interfaces,
-                        reverse=False):
-        """Propagate with GROMACS from the current system configuration.
+                        msg_file, reverse=False):
+        """
+        Propagate with GROMACS from the current system configuration.
 
         Here, we assume that this method is called after the propagate()
         has been called in the parent. The parent is then responsible
@@ -108,8 +96,11 @@ class GromacsEngine2(GromacsEngine):
             The object used for calculating the order parameter.
         interfaces : list of floats
             These interfaces define the stopping criterion.
-        reverse : boolean
-            If True, the system will be propagated backwards in time.
+        msg_file : object like :py:class:`.FileIO`
+            An object we use for writing out messages that are useful
+            for inspecting the status of the current propagation.
+        reverse : boolean, optional
+            If True, the system will be propagated backward in time.
 
         Returns
         -------
@@ -117,6 +108,7 @@ class GromacsEngine2(GromacsEngine):
             This is True if we generated an acceptable path.
         status : string
             A text description of the current status of the propagation.
+
         """
         status = 'propagating with GROMACS (reverse = {})'.format(reverse)
         logger.debug(status)
@@ -125,11 +117,16 @@ class GromacsEngine2(GromacsEngine):
         # Dumping of the initial config were done by the parent, here
         # we will just use it:
         initial_conf = system.particles.get_pos()[0]
-        # Get current order parameter
+        # Get the current order parameter:
         order = self.calculate_order(order_function, system)
+        msg_file.write(
+            '# Initial order parameter: {}'.format(
+                ' '.join(['{}'.format(i) for i in order])
+            )
+        )
         # So, here we will just blast off GROMACS and check the .trr
         # output when we can.
-        # 1) Create mdp_file with updated number of steps
+        # 1) Create mdp_file with updated number of steps:
         settings = {'gen_vel': 'no',
                     'nsteps': path.maxlen * self.subcycles,
                     'continuation': 'no'}
@@ -138,7 +135,7 @@ class GromacsEngine2(GromacsEngine):
                            delim='=')
         # 2) Run GROMACS preprocessor:
         out_files = self._execute_grompp(mdp_file, initial_conf, name)
-        # Generate some names that will be created by mdrun.
+        # Generate some names that will be created by mdrun:
         confout = '{}.{}'.format(name, self.ext)
         out_files['conf'] = confout
         out_files['cpt_prev'] = '{}_prev.cpt'.format(name)
@@ -155,51 +152,143 @@ class GromacsEngine2(GromacsEngine):
         cmd = shlex.split(self.mdrun.format(tpr_file, name, confout))
         # 3) Fire off GROMACS mdrun:
         logger.debug('Executing GROMACS.')
-        gro = GromacsRunner(cmd, trr_file, edr_file, self.exe_dir)
-        gro.start()  # Start GROMACS run.
-        for i, data in enumerate(gro.get_gromacs_frames()):
-            system.particles.pos = data['x']
-            if 'v' in data:
-                system.particles.vel = data['v']
-                if reverse:
-                    system.particles.vel *= -1
-            else:
-                system.particles.vel = None
-            length = box_matrix_to_list(data['box'])
-            system.update_box(length)
-            order = order_function.calculate_all(system)
-            phase_point = {'order': order,
-                           'pos': (trr_file, i),
-                           'vel': reverse,
-                           'vpot': None,
-                           'ekin': None}
-            status, success, stop, _ = self.add_to_path(path, phase_point,
-                                                        left, right)
-            if stop:
-                logger.debug('Ending propagate at %i. Reason: %s', i, status)
-                break
-        # Ask GROMACS to stop and close open files
-        gro.stop()
+        msg_file.write('# Trajectory file is: {}'.format(trr_file))
+        msg_file.write('# Starting GROMACS.')
+        msg_file.write('# Step order parameter cv1 cv2 ...')
+        with GromacsRunner(cmd, trr_file, edr_file, self.exe_dir) as gro:
+            for i, data in enumerate(gro.get_gromacs_frames()):
+                # Update the configuration file:
+                system.particles.set_pos((trr_file, i))
+                # Also provide the loaded positions since they are
+                # available:
+                system.particles.pos = data['x']
+                if 'v' in data:
+                    system.particles.vel = data['v']
+                    if reverse:
+                        system.particles.vel *= -1
+                else:
+                    system.particles.vel = None
+                length = box_matrix_to_list(data['box'])
+                system.update_box(length)
+                order = order_function.calculate(system)
+                msg_file.write(
+                    '{} {}'.format(
+                        i, ' '.join(['{}'.format(j) for j in order])
+                    )
+                )
+                snapshot = {'order': order,
+                            'config': (trr_file, i),
+                            'vel_rev': reverse}
+                phase_point = self.snapshot_to_system(system, snapshot)
+                status, success, stop, _ = self.add_to_path(path, phase_point,
+                                                            left, right)
+                if stop:
+                    logger.debug('Ending propagate at %i. Reason: %s',
+                                 i, status)
+                    break
         logger.debug('GROMACS propagation done, obtaining energies!')
+        msg_file.write('# Propagation done.')
+        msg_file.write('# Reading energies from: {}'.format(out_files['edr']))
         energy = self.get_energies(out_files['edr'])
-        if len(energy['potential']) < len(path.pos):
-            logger.warning('GROMACS energies were too short!')
-            path.vpot = np.zeros(len(path.pos))
-            path.ekin = np.zeros(len(path.pos))
-            path.vpot[:len(energy['potential'])] = energy['potential']
-            path.ekin[:len(energy['kinetic en.'])] = energy['kinetic en.']
-        else:
-            path.vpot = energy['potential'][:len(path.pos)]
-            path.ekin = energy['kinetic en.'][:len(path.pos)]
+        path.update_energies(energy['kinetic en.'], energy['potential'])
         logger.debug('Removing GROMACS output after propagate.')
         remove = [val for key, val in out_files.items() if key not in ('trr',)]
         self._remove_files(self.exe_dir, remove)
         self._remove_gromacs_backup_files(self.exe_dir)
         return success, status
 
+    def integrate(self, system, steps, order_function=None, thermo='full'):
+        """
+        Perform several integration steps.
+
+        This method will perform several integration steps using
+        GROMACS. It will also calculate order parameter(s) and energy
+        terms if requested.
+
+        Parameters
+        ----------
+        system : object like :py:class:`.System`
+            The system we are integrating.
+        steps : integer
+            The number of steps we are going to perform. Note that we
+            do not integrate on the first step (e.g. step 0) but we do
+            obtain the other properties. This is to output the starting
+            configuration.
+        order_function : object like :py:class:`.OrderParameter`, optional
+            An order function can be specified if we want to
+            calculate the order parameter along with the simulation.
+        thermo : string, optional
+            Select the thermodynamic properties we are to obtain.
+
+        Yields
+        ------
+        results : dict
+            The results from a MD step. This contains the state of the system
+            and order parameter(s) and energies (if calculated).
+
+        """
+        logger.debug('Integrating using GROMACS')
+        # Dump the initial config:
+        initial_file = self.dump_frame(system)
+        self.energy_terms = self.select_energy_terms(thermo)
+        if order_function:
+            order = self.calculate_order(order_function, system)
+        else:
+            order = None
+        name = 'pyretis-gmx'
+        # 1) Create mdp_file with updated number of steps:
+        # Note the -1 here due do different numbering in GROMACS and PyRETIS.
+        settings = {'nsteps': (steps - 1) * self.subcycles,
+                    'continuation': 'no'}
+        mdp_file = os.path.join(self.exe_dir, '{}.mdp'.format(name))
+        self._modify_input(self.input_files['input'], mdp_file, settings,
+                           delim='=')
+        # 2) Run GROMACS preprocessor:
+        out_files = self._execute_grompp(mdp_file, initial_file, name)
+        # Generate some names that will be created by mdrun:
+        confout = '{}.{}'.format(name, self.ext)
+        out_files['conf'] = confout
+        out_files['cpt_prev'] = '{}_prev.cpt'.format(name)
+        for key in ('cpt', 'edr', 'log', 'trr'):
+            out_files[key] = '{}.{}'.format(name, key)
+        # Remove some of these files if present (e.g. left over from a
+        # crashed simulation). This is so that GromacsRunner will not
+        # start reading a .trr left from a previous simulation.
+        remove = [out_files[key] for key in out_files if key not in ('tpr',)]
+        self._remove_files(self.exe_dir, remove)
+        tpr_file = out_files['tpr']
+        trr_file = os.path.join(self.exe_dir, out_files['trr'])
+        edr_file = os.path.join(self.exe_dir, out_files['edr'])
+        cmd = shlex.split(self.mdrun.format(tpr_file, name, confout))
+        # 3) Fire off GROMACS mdrun:
+        logger.debug('Executing GROMACS.')
+        with GromacsRunner(cmd, trr_file, edr_file, self.exe_dir) as gro:
+            for i, data in enumerate(gro.get_gromacs_frames()):
+                system.particles.pos = data['x']
+                if 'v' in data:
+                    system.particles.vel = data['v']
+                else:
+                    system.particles.vel = None
+                length = box_matrix_to_list(data['box'])
+                system.update_box(length)
+                results = {}
+                if order:
+                    results['order'] = order
+                if order_function:
+                    order = order_function.calculate(system)
+                time1 = (i * self.timestep * self.subcycles -
+                         0.1 * self.timestep)
+                time2 = ((i + 1) * self.timestep * self.subcycles +
+                         0.1 * self.timestep)
+                energy = self.get_energies(out_files['edr'], begin=time1,
+                                           end=time2)
+                results['thermo'] = self.rename_energies(energy)
+                yield results
+        logger.debug('GROMACS execution done.')
+
 
 def get_data(fileh, header):
-    """Read data from the trr file.
+    """Read data from the TRR file.
 
     Parameters
     ----------
@@ -214,6 +303,7 @@ def get_data(fileh, header):
         The data read from the file.
     data_size : integer
         The size of the data read.
+
     """
     data_size = sum([header[key] for key in TRR_DATA_ITEMS])
     data = read_trr_data(fileh, header)
@@ -240,6 +330,7 @@ def reopen_file(filename, fileh, inode, bytes_read):
         The new file object.
     out[1] : integer or None
         The new inode.
+
     """
     if os.stat(filename).st_ino != inode:
         new_fileh = open(filename, 'rb')
@@ -251,7 +342,7 @@ def reopen_file(filename, fileh, inode, bytes_read):
 
 
 def read_remaining_trr(filename, fileh, start):
-    """Read remaining frames from the trr.
+    """Read remaining frames from the TRR file.
 
     Parameters
     ----------
@@ -264,6 +355,13 @@ def read_remaining_trr(filename, fileh, start):
 
     Yields
     ------
+    out[0] : string
+        The header read from the file
+    out[1] : dict
+        The data read from the file.
+    out[2] : integer
+        The size of the data read.
+
     """
     stop = False
     bytes_read = start
@@ -290,20 +388,23 @@ def read_remaining_trr(filename, fileh, start):
                     bytes_read += new_bytes
                     yield header, data, bytes_read
             except EOFError:
-                # Hopefully, this code should not be reached...
+                # Hopefully, this code should not be reached.
                 stop = True
                 continue
 
 
-class GromacsRunner():
+class GromacsRunner:
     """A helper class for running GROMACS.
 
+    This class handles the reading of the TRR on the fly and
+    it is used to decide when to end the GROMACS execution.
+
     Attributes
     ----------
     cmd : string
-        The command for executing GROMACS
+        The command for executing GROMACS.
     trr_file : string
-        The .trr file we are going to read.
+        The GROMACS TRR file we are going to read.
     edr_file : string
         A .edr file we are going to read.
     exe_dir : string
@@ -313,35 +414,37 @@ class GromacsRunner():
     running : None or object like :py:class:`subprocess.Popen`
         The process running GROMACS.
     bytes_read : integer
-        The number of bytes we have read so far from the .trr file.
-    inode : integer
-        The current inode we are using.
+        The number of bytes read so far from the TRR file.
+    ino : integer
+        The current inode we are using for the file.
     stop_read : boolean
         If this is set to True, we will stop the reading.
     SLEEP : float
         How long we wait after an unsuccessful read before
         reading again.
     data_size : integer
-        The size of the data (x, v, f, box, etc.) in the .trr file.
+        The size of the data (x, v, f, box, etc.) in the TRR file.
     header_size : integer
-        The size of the header in the .trr file.
+        The size of the header in the TRR file.
+
     """
 
     SLEEP = 0.1
 
     def __init__(self, cmd, trr_file, edr_file, exe_dir):
-        """Just set up command and files.
+        """Set-up the GROMACS command and the files we need.
 
         Parameters
         ----------
         cmd : string
-            The command for executing GROMACS
+            The command for executing GROMACS.
         trr_file : string
-            The .trr file we are going to read.
+            The GROMACS TRR file we are going to read.
         edr_file : string
             A .edr file we are going to read.
         exe_dir : string
             Path to where we are currently running GROMACS.
+
         """
         self.cmd = cmd
         self.trr_file = trr_file
@@ -354,27 +457,50 @@ class GromacsRunner():
         self.stop_read = True
         self.data_size = 0
         self.header_size = 0
+        self.stdout_name = None
+        self.stderr_name = None
+        self.stdout = None
+        self.stderr = None
 
     def start(self):
         """Start execution of GROMACS and wait for output file creation."""
         logger.debug('Starting GROMACS execution in %s', self.exe_dir)
+
+        self.stdout_name = os.path.join(self.exe_dir, 'stdout.txt')
+        self.stderr_name = os.path.join(self.exe_dir, 'stderr.txt')
+        self.stdout = open(self.stdout_name, 'wb')
+        self.stderr = open(self.stderr_name, 'wb')
+
         self.running = subprocess.Popen(
-            self.cmd, stdin=subprocess.PIPE,
-            stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
+            self.cmd,
+            stdin=subprocess.PIPE,
+            stdout=self.stdout,
+            stderr=self.stderr,
             shell=False,
             cwd=self.exe_dir
         )
-        # Wait for the trr/edr file to appear
+        present = []
+        # Wait for the TRR/EDR files to appear:
         for fname in (self.trr_file, self.edr_file):
             while not os.path.isfile(fname):
                 logger.debug('Waiting for GROMACS file "%s"', fname)
                 sleep(self.SLEEP)
-        # Prepare and open the trr file:
+                poll = self.check_poll()
+                if poll is not None:
+                    logger.debug('GROMACS execution stopped')
+                    break
+            if os.path.isfile(fname):
+                present.append(fname)
+        # Prepare and open the TRR file:
         self.bytes_read = 0
-        self.fileh = open(self.trr_file, 'rb')
-        self.ino = os.fstat(self.fileh.fileno()).st_ino
-        self.stop_read = False
+        # Ok, so GROMACS might have crashed in between writing the
+        # files. Check that both files are indeed here:
+        if self.trr_file in present and self.edr_file in present:
+            self.fileh = open(self.trr_file, 'rb')
+            self.ino = os.fstat(self.fileh.fileno()).st_ino
+            self.stop_read = False
+        else:
+            self.stop_read = True
 
     def __enter__(self):
         """Start running GROMACS, for a context manager."""
@@ -382,10 +508,11 @@ class GromacsRunner():
         return self
 
     def get_gromacs_frames(self):
-        """Read the GROMACS .trr file on-the-fly."""
+        """Read the GROMACS TRR file on-the-fly."""
         first_header = True
+        header = None
         while not self.stop_read:
-            poll = self.running.poll()
+            poll = self.check_poll()
             if poll is not None:
                 # GROMACS is done, read remaining data.
                 self.stop_read = True
@@ -396,7 +523,7 @@ class GromacsRunner():
                         yield data
 
             else:
-                # first we try to get the header from the file:
+                # First we try to get the header from the file:
                 size = os.path.getsize(self.trr_file)
                 if self.header_size == 0:
                     header_size = TRR_HEAD_SIZE
@@ -420,7 +547,7 @@ class GromacsRunner():
                         if first_header:
                             logger.debug('TRR header was: %i', new_bytes)
                             first_header = False
-                        # calculate the size of the data
+                        # Calculate the size of the data:
                         self.data_size = sum([header[key] for key in
                                               TRR_DATA_ITEMS])
                         data = None
@@ -440,16 +567,16 @@ class GromacsRunner():
                                         self.fileh = new_fileh
                                         self.ino = new_ino
                                 if data is None:
-                                    # Data is not ready, will wait:
+                                    # Data is not ready, just wait:
                                     sleep(self.SLEEP)
                                 else:
                                     self.bytes_read += new_bytes
                                     yield data
                             else:
-                                # Data is not ready, will wait:
+                                # Data is not ready, just wait:
                                 sleep(self.SLEEP)
                 else:
-                    # header was not ready, just wait before trying again...
+                    # Header was not ready, just wait before trying again.
                     sleep(self.SLEEP)
 
     def close(self):
@@ -457,17 +584,27 @@ class GromacsRunner():
         if self.fileh is not None and not self.fileh.closed:
             logger.debug('Closing GROMACS file: "%s"', self.trr_file)
             self.fileh.close()
+        for handle in (self.stdout, self.stderr):
+            if handle is not None and not handle.closed:
+                handle.close()
 
     def stop(self):
         """Stop the current GROMACS execution."""
-        if self.running is not None:
+        if self.running:
+            for handle in (self.running.stdin, self.running.stdout,
+                           self.running.stderr):
+                if handle:
+                    try:
+                        handle.close()
+                    except AttributeError:
+                        pass
             if self.running.returncode is None:
                 logger.debug('Terminating GROMACS execution')
                 self.running.terminate()
                 logger.debug('Waiting for GROMACS termination')
-                self.running.wait(timeout=360)
+            self.running.wait(timeout=360)
         self.stop_read = True
-        self.close()  # close trr file.
+        self.close()  # Close the TRR file.
 
     def __exit__(self, exc_type, exc_val, exc_tb):
         """Just stop execution and close file for a context manager."""
@@ -476,3 +613,17 @@ class GromacsRunner():
     def __del__(self):
         """Just stop execution and close file."""
         self.stop()
+
+    def check_poll(self):
+        """Check the current status of the running subprocess."""
+        if self.running:
+            poll = self.running.poll()
+            if poll is not None:
+                logger.debug('Execution of GROMACS stopped')
+                logger.debug('Return code was: %i', poll)
+                if poll != 0:
+                    logger.error('STDOUT, see file: %s', self.stdout_name)
+                    logger.error('STDERR, see file: %s', self.stderr_name)
+                    raise RuntimeError('Error in GROMACS execution.')
+            return poll
+        raise RuntimeError('GROMACS is not running.')
