@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2015, PyRETIS Development Team.
+# Copyright (c) 2019, PyRETIS Development Team.
 # Distributed under the LGPLv2.1+ License. See LICENSE for more info.
 """This module defines common methods for the settings handling.
 
@@ -35,7 +35,7 @@ from pyretis.engines import engine_factory
 from pyretis.orderparameter import order_factory
 from pyretis.orderparameter.orderparameter import CompositeOrderParameter
 from pyretis.forcefield.factory import potential_factory
-logger = logging.getLogger(__name__)  # pylint: disable=C0103
+logger = logging.getLogger(__name__)  # pylint: disable=invalid-name
 logger.addHandler(logging.NullHandler())
 
 
@@ -44,7 +44,7 @@ __all__ = ['create_external', 'check_settings', 'import_from',
 
 
 def import_from(module_path, function_name):
-    """Method to import a method/class from a module.
+    """Import a method/class from a module.
 
     This method will dynamically import a specified method/object
     from a module and return it. If the module can not be imported or
@@ -72,6 +72,7 @@ def import_from(module_path, function_name):
     References
     ----------
     .. [#] http://bugs.python.org/issue21436
+
     """
     try:
         module_name = os.path.basename(module_path)
@@ -87,6 +88,7 @@ def import_from(module_path, function_name):
             )
             module = importlib.util.module_from_spec(spec)
             spec.loader.exec_module(module)
+            sys.modules[module_name] = module
         logger.debug('Imported module: %s', module)
         return getattr(module, function_name)
     except (ImportError, IOError):
@@ -105,8 +107,8 @@ def check_settings(settings, required):
 
     This method will look for required settings in the given
     `settings`. If one or more keys from the given `required` list of
-    strings are not found, this method will return False. Otherwise
-    if will return True. Typically, and exception should be raised if
+    strings are not found, this method will return False. Otherwise,
+    it will return True. Typically, an exception should be raised if
     False is returned, this is handled outside the method in case
     someone wants to add some magic handling of missing settings.
 
@@ -124,6 +126,7 @@ def check_settings(settings, required):
         True if all required settings are present, False otherwise.
     not_found : list of strings
         There are the required settings we did not find.
+
     """
     result = True
     not_found = []
@@ -136,9 +139,9 @@ def check_settings(settings, required):
 
 def create_external(settings, key, factory, required_methods,
                     key_settings=None):
-    """Method to create objects from settings.
+    """Create external objects from settings.
 
-    This method will handle creation of objects from settings. The
+    This method will handle the creation of objects from settings. The
     requested objects can be PyRETIS internals or defined in external
     modules.
 
@@ -154,8 +157,8 @@ def create_external(settings, key, factory, required_methods,
     required_methods : list of strings
         The methods we need to have if creating an object from external
         files.
-    key_settings : dict
-        This dictionary contains the settings for specific key we
+    key_settings : dict, optional
+        This dictionary contains the settings for the specific key we
         are processing. If this is not given, we will try to obtain
         these settings by `settings[key]`. The reason why we make it
         possible to pass these as settings is in case we are processing
@@ -168,6 +171,7 @@ def create_external(settings, key, factory, required_methods,
     -------
     out : object
         This object represents the class we are requesting here.
+
     """
     if key_settings is None:
         try:
@@ -180,45 +184,44 @@ def create_external(settings, key, factory, required_methods,
     try:
         klass = key_settings['class']
     except KeyError:
-        msg = 'No "{}" setting "class" specified!'.format(key)
-        logger.critical(msg)
-        raise ValueError(msg)
+        logger.debug('No "class" setting for "%s" specified. Skipping set-up',
+                     key)
+        return None
     if module is None:
         return factory(key_settings)
+    # Here we assume we are to load from a file. Before we import
+    # we need to check that the path is ok or if we should include
+    # the 'exe-path' from settings.
+    # 1) Check if we can find the module:
+    if os.path.isfile(module):
+        obj = import_from(module, klass)
     else:
-        # Here we assume we are to load from a file. Before we import
-        # we need to check that the path is ok or if we should include
-        # the 'exe-path' from settings.
-        # 1) Check if we can find the module:
-        if os.path.isfile(module):
+        if 'exe-path' in settings['simulation']:
+            module = os.path.join(settings['simulation']['exe-path'],
+                                  module)
             obj = import_from(module, klass)
         else:
-            if 'exe-path' in settings['simulation']:
-                module = os.path.join(settings['simulation']['exe-path'],
-                                      module)
-                obj = import_from(module, klass)
-            else:
-                msg = 'Could not find module "{}" for {}!'.format(module, key)
-                raise ValueError(msg)
-        # run some checks:
-        for function in required_methods:
-            objfunc = getattr(obj, function, None)
-            if not objfunc:
-                msg = 'Could not find method {}.{}'.format(klass,
-                                                           function)
+            msg = 'Could not find module "{}" for {}!'.format(module, key)
+            raise ValueError(msg)
+    # run some checks:
+    for function in required_methods:
+        objfunc = getattr(obj, function, None)
+        if not objfunc:
+            msg = 'Could not find method {}.{}'.format(klass,
+                                                       function)
+            logger.critical(msg)
+            raise ValueError(msg)
+        else:
+            if not callable(objfunc):
+                msg = 'Method {}.{} is not callable!'.format(klass,
+                                                             function)
                 logger.critical(msg)
                 raise ValueError(msg)
-            else:
-                if not callable(objfunc):
-                    msg = 'Method {}.{} is not callable!'.format(klass,
-                                                                 function)
-                    logger.critical(msg)
-                    raise ValueError(msg)
-        return initiate_instance(obj, key_settings)
+    return initiate_instance(obj, key_settings)
 
 
 def create_orderparameter(settings):
-    """Method to create order parameters from settings.
+    """Create order parameters from settings.
 
     Parameters
     ----------
@@ -229,13 +232,17 @@ def create_orderparameter(settings):
     -------
     out : object like :py:class:`.OrderParameter`
         This object represents the order parameter.
+
     """
     main_order = create_external(
         settings,
         'orderparameter',
         order_factory,
-        ('calculate',),
+        ['calculate'],
     )
+    if main_order is None:
+        logger.info('No order parameter created')
+        return None
     logger.info('Created main order parameter:\n%s', main_order)
 
     extra_cv = []
@@ -245,7 +252,7 @@ def create_orderparameter(settings):
             settings,
             'collective-variable',
             order_factory,
-            ('calculate',),
+            ['calculate'],
             key_settings=order_setting
         )
         logger.info('Created additional collective variable:\n%s', order)
@@ -259,7 +266,7 @@ def create_orderparameter(settings):
 
 
 def create_engine(settings):
-    """Method to create an engine from settings.
+    """Create an engine from settings.
 
     Parameters
     ----------
@@ -270,13 +277,17 @@ def create_engine(settings):
     -------
     out : object like :py:class:`.EngineBase`
         This object represents the engine.
+
     """
-    return create_external(settings, 'engine', engine_factory,
-                           ['integration_step'])
+    engine = create_external(settings, 'engine', engine_factory,
+                             ['integration_step'])
+    if not engine:
+        raise ValueError('Could not create engine from settings!')
+    return engine
 
 
 def create_potential(settings, key_settings):
-    """Method to create a potential from settings.
+    """Create a potential from settings.
 
     Parameters
     ----------
@@ -289,6 +300,7 @@ def create_potential(settings, key_settings):
     -------
     out : object like :py:class:`.PotentialFunction`
         The object representing the potential function.
+
     """
     return create_external(settings, 'potential', potential_factory,
                            ['force', 'potential', 'potential_and_force'],
